// Distributed Objects
// Create by Chris Bartlett 20.02.2005
// Permission is granted for all forms of reuse

// Usage notes
// -----------
// This example illustrates how to keep a list of positions in an array and 
// check that subsequent positions are not within a given minimum distance of
// previously used positions.
// Warnings
// - The closer you get to the maximum number of objects that can be crammed
//   into the space the less efficient it gets.
// - If you give it a space too small to fit enough objects it'll loop forever.

#include "shapes.inc" 

camera {location<-3,4,-12> look_at<0,0,0>}
light_source{<30,30,-20> color rgb 2}  
//light_source{<-20,10,-25> color rgb 2}  


#macro ppCrowdPositions(ppPlatformObject,ppNumberOfPeople,ppSafetyZone,ppCrowdRand,ppMaxTries)
  #declare ppPeoplePositions = array[ppNumberOfPeople];

  #local ppPlatformMin    = <0,0,0>;
  #local ppPlatformMax    = <0,0,0>;
  #local ppPlatformNormal = <0,0,0>;

  Extents(ppPlatformObject, ppPlatformMin, ppPlatformMax)

  #local ppCrowdExtents = ppPlatformMax - ppPlatformMin;
  #local ppStartTraceHeight = 2*ppPlatformMax.y;
  
  #local ppI=0;
  #local ppTryCount=0;
  #local ppTraceVector = <0,-1,0>;
  #while (ppI<ppNumberOfPeople & ppTryCount<ppMaxTries)
    // Pick a random position on the platform
    #local ppIntersectionNormal = <0,0,0>;
    #while (vlength(ppIntersectionNormal)=0 & ppTryCount<ppMaxTries)
      #local ppTraceStart = <rand(ppCrowdRand),0,rand(ppCrowdRand)>*ppCrowdExtents+ppPlatformMin*<1,0,1>+ppStartTraceHeight*y; 
      #declare ppIntersectionPoint = trace ( ppPlatformObject, ppTraceStart, ppTraceVector, ppIntersectionNormal ); 
      #local ppTryCount = ppTryCount + 1;
    #end
    #if (ppTryCount<ppMaxTries) 
      #declare ppPeoplePositions[ppI]= ppIntersectionPoint; 
      // Check the new position against any we've already used
      #local ppTooClose = 0;
      #local ppJ = 0;
      #while (ppJ<ppI)
        // See if the length of the vector joining the two positions is less than required distance 
        #if (vlength(ppPeoplePositions[ppI]-ppPeoplePositions[ppJ])<ppSafetyZone) 
          #local ppTooClose = 1; 
          #local ppJ=ppI;                           // Shortcut out of the loop
        #end
        #local ppJ = ppJ + 1;
      #end
      // If we didn't find any too close then we use this position, otherwise we go round again
      #if (ppTooClose=0)  
        #local ppI=ppI+1;  
      #end 
    #else #local ppNumberOfPeople = ppI;
    #end 
  #end 
  ppNumberOfPeople
#end

#macro ppCrowd(ppPlatformObject,ppSafetyZone,ppCrowdRand,ppMaxTries)
  #declare ppNumberOfPeople = ppCrowdPositions(ppPlatformObject,ppNumberOfPeople,ppSafetyZone,ppCrowdRand,ppMaxTries);
  #local ppI = 0;
  #while (ppI<ppNumberOfPeople)
    object {ppPersonObject
      rotate (rand(ppCrowdRand)*y)*360
      translate ppPeoplePositions[ppI] 
      pigment {color rgb <rand(ppCrowdRand),rand(ppCrowdRand),rand(ppCrowdRand)>}
    }
    #local ppI = ppI + 1;
  #end
#end

#declare ppSafetyZone = 0.5;                     // Radius of the safety zone around an object
#declare ppCrowdRand = seed(42);                 // Any number will do 
#declare ppPlatformObject = union {
  box {<-5,-1,-8><8,0,5> rotate y*45}
  #local ppI = 1;
  #while (ppI<9)
    box {<-2,-0.2+ppI*0.2,-8+0.4*ppI><2,0+ppI*0.2,0>}
    #local ppI = ppI + 1;
  #end 
  cylinder {<0,-1,0><0,1.6,0>,4}
  pigment {color rgb <1,1,1>}
}
#declare ppPersonObject   = cylinder {0,1.7*y,0.05}   
#declare ppMaxTries = 500;

#declare ppNumberOfPeople = 100;
#declare myCrowd = ppCrowd(ppPlatformObject,ppSafetyZone,ppCrowdRand,ppMaxTries)

object {myCrowd}
object {ppPlatformObject}