/***************************************************************************************************/
/* pppovperson.inc  - Copyright Chris Bartlett Oct 2004.                                           */
/* This work is licensed for royalty free re-use under the terms of the POV-Person License.        */
/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */
/***************************************************************************************************/

#version 3.5;
#declare POVPerson = "V2.0";
#include "transforms.inc"
//#include "arrays.inc"
#ifndef(ppCamera) #declare ppCamera="User Defined"; #end  
#debug concat("'",ppCamera,"' camera.\n")
#declare ppRandomStream = seed(1);


// The ppFigure macro is the principal control macro for the POV-Person library. It defines a 
// figure based upon a series of control files that are specified as parameters to the ppFigure
// call.
//
// Parameters:
// -----------
//     ppCharacter        - The file used to define the character to be drawn
//     ppStyle            - The file containing style parameters
//     ppExpression       - The file containing expression parameters
//     ppPose             - The file containing pose parameters
//     ppCostume          - The file controling clothing and other accessories

#macro ppFigure(ppCharacter,ppStyle,ppExpression,ppPose,ppCostume) 
  ppInitialiseMatrices()
  
  #include ppCharacter
  #include ppStyle
  #include ppExpression

  #include concat(ppCharacterTemplate,".inc")      // e.g. pphuman001.inc  

  #ifdef (ppLeftEyeRotation ) ppRotateJoint(ppLeftEyeJointRef ,ppLeftEyeRotation.x ,ppLeftEyeRotation.y ,ppLeftEyeRotation.z ) #end      
  #ifdef (ppRightEyeRotation) ppRotateJoint(ppRightEyeJointRef,ppRightEyeRotation.x,ppRightEyeRotation.y,ppRightEyeRotation.z) #end      
//  #ifdef (ppExpressionMacro) ppExpressionMacro() #end
  #include ppPose

  
  #ifndef(ppFixedBodyPart           ) #declare ppFixedBodyPart            = ppLeftFootRef;  #end
  #ifndef(ppFixedBodyPartPosition   ) #declare ppFixedBodyPartPosition    = <0,0,0>;        #end
  #ifndef(ppFixedBodyPartOrientation) #declare ppFixedBodyPartOrientation = <0,0,0>;        #end
  
  #declare ppBodyPartListPosition[ppFixedBodyPart]   =ppFixedBodyPartPosition;
  #declare ppBodyPartListOrientation[ppFixedBodyPart]=ppFixedBodyPartOrientation;

  ppCalculateBodyPartTransformations()
  
  #if (strcmp(ppCostume,"none")>0)
    #include concat(ppCharacterTemplate,"accessories.inc")
    #include ppCostume
    #ifdef (ppCostumeItems)
      #local ppI = 0;
      #while (ppI<dimension_size(ppCostumeItems,1))
        #include ppCostumeItems[ppI]
        #local ppI = ppI + 1; 
      #end 
    #end 
  #end  
  
  #ifdef (ppLookAt) ppAdjustEyes() #end
  
  ppBuildFigure()
#end 


// The ppCrowd macro positions a series of figures at a set of positions that have been predefined 
// and stored in an array.
//
// Parameters:
// -----------
//     ppFirstPosition    - First position to use
//     ppNumberOfFigures  - The number of figures to position

#macro ppCrowd(ppFirstPosition, ppNumberOfFigures)
  #ifndef(ppCrowdFigureMinRotation) #declare ppCrowdFigureMinRotation =   0;  #end   //  Minimum amount to rotate a figure by
  #ifndef(ppCrowdFigureMaxRotation) #declare ppCrowdFigureMaxRotation = 360;  #end   //  Maximum amount to rotate a figure by
  #ifndef(ppCrowdRand)              #declare ppCrowdRand = seed(21); #end
  #local ppI = 0;
  #local ppFigureListDim = dimension_size(ppCrowdFigureList,1);
  union {
    #while (ppI<ppNumberOfFigures)
      object {ppCrowdFigureList[mod(ppI,ppFigureListDim)]
        rotate y*(ppCrowdFigureMinRotation+rand(ppCrowdRand)*(ppCrowdFigureMaxRotation-ppCrowdFigureMinRotation))
        translate ppFigurePositions[ppI+ppFirstPosition] 
      }
      #local ppI = ppI + 1;
    #end
  }
#end


// The ppCrowdPositions macro takes an object and finds a set of points on its surface that it stores
// in an array. The points are randomised, but are all found by projecting a ray down vertically onto
// the object and are selected to make sure they are <ol><li> not too close to other points, <li> that the surface
// is not too steep and <li> that the point is not too close to an edge.</ol>
//
// Parameters:
// -----------
//     ppNewFigures       - The number of figures required

#macro ppCrowdPositions(ppNewFigures)
  #include "shapes.inc" 
  

//  #local ppCrowdRand = seed(ppCrowdSeed);

  #ifndef(ppCrowdRand)                 #declare ppCrowdRand = seed(38);                     #end
  #ifndef(ppCrowdPositionMaxTries)     #declare ppCrowdPositionMaxTries = 100;              #end   //  Control number to avoid infinite loops if the surface is too small for the number of positions requested.
  #ifndef(ppFigurePositions)           #declare ppFigurePositions = array[ppNewFigures];    #end
  #ifndef(ppFigureCounter)             #declare ppFigureCounter = 0;                        #end

  #ifndef(ppCrowdFigureSeparation    ) #declare ppCrowdFigureSeparation     = 0.5;          #end   //  Minimum separation between the centres of any two positions
  #ifndef(ppCrowdDistanceFromEdge    ) #declare ppCrowdDistanceFromEdge     = 0.25;         #end   //  Minimum distance of a position from an edge on the objects surface
  #ifndef(ppCrowdPermittedDrop       ) #declare ppCrowdPermittedDrop        = 0.1;          #end   //  Maximum change of height before it's considered an edge
  #ifndef(ppCrowdEdgeDetectionSamples) #declare ppCrowdEdgeDetectionSamples = 4;            #end   //  Number of samples taken around a position when looking for edges
  #ifndef(ppCrowdPlatform)             #declare ppCrowdPlatform   = box{<0,-0.001,0>,<10,0,10>} #end   //  Default surface is a 10 unit square starting at the origin

  #local ppPlatformMin    = <0,0,0>;
  #local ppPlatformMax    = <0,0,0>;
  #local ppPlatformNormal = <0,0,0>;

  Extents(ppCrowdPlatform, ppPlatformMin, ppPlatformMax)

  #local ppCrowdExtents = ppPlatformMax - ppPlatformMin;
  #local ppStartTraceHeight = 2*ppPlatformMax.y;   
  
  #local ppI=0;
  #local ppTryCount=0;
  #local ppTotalTryCount=0;
  #local ppTraceVector = <0,-1,0>;
  #local ppRimIntersectionNormal = <0,0,0>;
  #while (ppI<ppNewFigures & ppTryCount<ppCrowdPositionMaxTries)
    // Pick a random position on the platform
    #local ppSurfaceNotGood = 0;
    #local ppIntersectionNormal = <0,0,0>;
    #while ((vlength(ppIntersectionNormal)=0 | ppSurfaceNotGood=1) & ppTryCount<ppCrowdPositionMaxTries) 
      #local ppTraceStart = <rand(ppCrowdRand),0,rand(ppCrowdRand)>*ppCrowdExtents+ppPlatformMin*<1,0,1>+ppStartTraceHeight*y; 
      #declare ppIntersectionPoint = trace ( ppCrowdPlatform, ppTraceStart, ppTraceVector, ppIntersectionNormal );
      #local ppSurfaceNotGood = 0;
      #if (vlength(ppIntersectionNormal)!=0) 
        #local ppJ = 1;  // Sample around the area to avoid getting too close to surface edges
        #while (ppJ <= ppCrowdEdgeDetectionSamples) 
          #local ppTraceRimStart = ppTraceStart + vrotate(x,y*ppJ*360/ppCrowdEdgeDetectionSamples)*ppCrowdDistanceFromEdge;     
          #local ppRimPoint = trace ( ppCrowdPlatform, ppTraceRimStart, ppTraceVector, ppRimIntersectionNormal );
          #if (vlength(ppIntersectionNormal)=0 | abs(ppIntersectionPoint.y-ppRimPoint.y) > ppCrowdPermittedDrop)
            #local ppSurfaceNotGood = 1;
          #end  
          #local ppJ = ppJ + 1;
        #end
      #end 
      #local ppTryCount = ppTryCount + 1;
      #local ppTotalTryCount = ppTotalTryCount + 1;
    #end
    #if (ppTryCount<ppCrowdPositionMaxTries)                  
      // Check the new position against any we've already used
      #local ppTooClose = 0;
      #local ppJ = 0;
      #while (ppJ<ppFigureCounter) 
        // See if the length of the vector joining the two positions is less than required distance 
        #if (vlength(ppIntersectionPoint-ppFigurePositions[ppJ])<ppCrowdFigureSeparation) 
          #local ppTooClose = 1; 
          #local ppJ=ppFigureCounter;                           // Shortcut out of the loop
        #end
        #local ppJ = ppJ + 1;
      #end
      // If we didn't find any too close then we use this position, otherwise we go round again
      #if (ppTooClose=0)  
        #declare ppFigurePositions[ppFigureCounter]= ppIntersectionPoint;        // Populate array starting at position 0
//  #debug concat("Intersection Point: ",vstr(3,ppIntersectionPoint,",",0,3),"\n")
        #declare ppFigureCounter = ppFigureCounter + 1;
        #local ppTryCount=0;
        #local ppI=ppI+1;  
      #end 
    #else #debug concat("ppCrowdPositions Macro: Failed to find sufficient viable positions within the specified number of tries (",str(ppCrowdPositionMaxTries,0,0),").\n")
    #end 
  #end 
  #debug concat("ppCrowdPositions Macro: Tested ",str(ppTotalTryCount,0,0)," positions. Found ",str(ppI,0,0),"/",str(ppNewFigures,0,0)," viable positions.","\n")
  ppI
#end



// This macro initialises a series of matrices used by POVPerson to define the relationship between the different parts of the
// figure. This is broken down into two principal groups: 
//
//  Body Parts - A set of matrices containing an object definition, a scale factor, 
//               a position and a rotation for the object.
// 
//  Joints     - A set of matrices containing the reference numbers of two body points,
//               the centre point associated with each object about which the other rotates, 
//               the lower and upper rotation limits in each direction, the actual <x,y,z> rotation of this joint.  
//               Note. The centre point of a joint is not necessarily within an object. For a ball joint the centre 
//               of the joint will be the centre of the ball on one of the objects and the centre of radius of the 
//               socket on the other, which will clearly not be inside the second object. 
//
// Parameters:
// -----------
//               - No parameters are passed to this macro. 

#macro ppInitialiseMatrices()
  // It is not sure how many body parts will be required, so we just define a reasonably large set of arrays.
  #declare ppMaxBodyParts = 300;
  #declare ppBodyPartListName        = array[ppMaxBodyParts];    // The name of the body part 
  #declare ppBodyPartListObject      = array[ppMaxBodyParts];    // The object definition 
  #declare ppBodyPartListScale       = array[ppMaxBodyParts];    // The scale vector to be applied to the object
  #declare ppBodyPartListOrientation = array[ppMaxBodyParts];    // The Rotation to be applied to orient the object (normally calculated)
  #declare ppBodyPartListPosition    = array[ppMaxBodyParts];    // The Position the object is to be moved to (normally calculated)
  #declare ppBodyPartCount = 0;
  
  #declare ppJointNames     = array[ppMaxBodyParts];       // The names of the joints;
  #declare ppJointObjects   = array[ppMaxBodyParts][2];    // The reference numbers of the 2 objects joined at this joint
  #declare ppJointCentres   = array[ppMaxBodyParts][2];    // The centre point of the joint for each object relative to its origin before scaling
  #declare ppJointAxes      = array[ppMaxBodyParts][3];    // Centre Line, Primary Rotational Axis and Secondary Axis for second object
  #declare ppJointLimits    = array[ppMaxBodyParts][3][2]; // Upper and Lower limits for each axis
  #declare ppJointRotations = array[ppMaxBodyParts][3];    // The rotations of the second object around the 3 axes defined for it
  #declare ppJointRotationStash  = array[ppMaxBodyParts][3];    // Somewhere to copy of the rotations for animation calculations
  #declare ppJointCount = 0;
#end


// This macro adds a body part to the arrays holding the list of body parts and their scale then returns a reference number
// that will be stored in a variable with a name indicating the body part concerned. 
// The position and rotation settings are done later. 
//
// Parameters:
// -----------
//     ppBodyPartName     - The name of the body part (for debugging purposes) 
//     ppObject           - The object to be added to the array of body parts  

#macro ppAddBodyPart (ppBodyPartName,ppObject)
  #declare ppBodyPartListName[ppBodyPartCount]   = ppBodyPartName; 
  #declare ppBodyPartListObject[ppBodyPartCount] = object {ppObject}; 
  #declare ppBodyPartListScale[ppBodyPartCount]  = <1,1,1>;
  ppBodyPartCount;
  #declare ppBodyPartCount = ppBodyPartCount+1;
#end  


// This macro sets a value that will be used to scale a body part. 
//
// Parameters:
// -----------
//     ppThisBodyPart     - The reference number of a body part that has previously been added to the array of body parts
//     ppScale            - A 3D vector defining the scale in x, y and z of this body part
#macro ppScaleBodyPart (ppThisBodyPart,ppScale)
  #declare ppBodyPartListScale[ppThisBodyPart]  = ppScale;
#end  


// This macro retrieves a body part from the arrays holding the list of body parts, scales, rotates and translates it and 
// returns it as an object. 
// This macro sets a value that will be used to scale a body part. 
//
// Parameters:
// -----------
//     ppI                - The reference number of a body part that has previously been added to the array of body parts
#macro ppGetBodyPart(ppI)
  object {ppBodyPartListObject[ppI] 
    scale     ppBodyPartListScale[ppI] 
    #ifdef(ppBodyPartListOrientation[ppI]) rotate    ppBodyPartListOrientation[ppI]  #end
    #ifdef(ppBodyPartListPosition[ppI])    translate ppBodyPartListPosition[ppI]     #end
  }
#end


// This macro is used to define a joint between two body parts referenced using two body part numbers returned from ppAddBodyPart()
// The joint centre for each object is relative to the origin for that object.
// The rotations vector is the difference in orientation between the first and second object. 
//
// Parameters:
// -----------
//     ppJointName               - The name of this joint - This is used when position files are generated
//     ppObjectRef1              - The reference number of a body part that has previously been added to the array of body parts
//     ppObjectRef2              - The reference number of a second body part that has previously been added to the array of body parts
//     ppCentrePoint1            - The position of the joint relative to the origin of the first object
//     ppCentrePoint2            - The position of the joint relative to the origin of the second object
//     ppJointCentralAxis        - The axis around which the second object would twist around the joint
//     ppJointPrimaryAxis        - The first axis of rotation of the second object around the position of the joint
//     ppJointSecondaryAxis      - The second axis of rotation of the second object around the position of the joint
//     ppCentralAxisLowerLimit   - The lowest angle in degrees that the second object can twist around the joint
//     ppCentralAxisUpperLimit   - The highest angle in degrees that the second object can twist around the joint
//     ppPrimaryAxisLowerLimit   - The lowest angle in degrees that the second object can rotate around the joint in the 
//                                 direction of the first axis
//     ppPrimaryAxisUpperLimit   - The highest angle in degrees that the second object can rotate around the joint in the 
//                                 direction of the first axis 
//     ppSecondaryAxisLowerLimit - The lowest angle in degrees that the second object can rotate around the joint in the 
//                                 direction of the second axis 
//     ppSecondaryAxisUpperLimit - The highest angle in degrees that the second object can rotate around the joint in the 
//                                 direction of the second axis    

#macro ppAddJoint (ppJointName,ppObjectRef1,ppObjectRef2,ppCentrePoint1,ppCentrePoint2,ppJointCentralAxis,ppJointPrimaryAxis,ppJointSecondaryAxis
         ,ppCentralAxisLowerLimit,ppCentralAxisUpperLimit
         ,ppPrimaryAxisLowerLimit,ppPrimaryAxisUpperLimit
         ,ppSecondaryAxisLowerLimit,ppSecondaryAxisUpperLimit)
  #declare ppJointNames[ppJointCount] = ppJointName;
  #declare ppJointObjects[ppJointCount][0] = ppObjectRef1;
  #declare ppJointObjects[ppJointCount][1] = ppObjectRef2;
  #declare ppJointCentres[ppJointCount][0] = ppCentrePoint1;
  #declare ppJointCentres[ppJointCount][1] = ppCentrePoint2; 
  #declare ppJointAxes[ppJointCount][0] = ppJointCentralAxis;
  #declare ppJointAxes[ppJointCount][1] = ppJointPrimaryAxis; 
  #declare ppJointAxes[ppJointCount][2] = ppJointSecondaryAxis; 
  #declare ppJointLimits[ppJointCount][0][0] =ppCentralAxisLowerLimit;
  #declare ppJointLimits[ppJointCount][0][1] =ppCentralAxisUpperLimit;
  #declare ppJointLimits[ppJointCount][1][0] =ppPrimaryAxisLowerLimit;
  #declare ppJointLimits[ppJointCount][1][1] =ppPrimaryAxisUpperLimit;
  #declare ppJointLimits[ppJointCount][2][0] =ppSecondaryAxisLowerLimit;
  #declare ppJointLimits[ppJointCount][2][1] =ppSecondaryAxisUpperLimit;
  #declare ppJointRotations[ppJointCount][0] = 0;
  #declare ppJointRotations[ppJointCount][1] = 0; 
  #declare ppJointRotations[ppJointCount][2] = 0; 
  ppJointCount;    // Return a reference to the joint just created   
  #declare ppJointCount = ppJointCount+1; 
#end  


// This macro is used to define a joint between two body parts referenced using two body part numbers returned from ppAddBodyPart(). 
// The Axes, Joint Centres and Joint Limits are set using a 'left-right' mirror of another joint, a reference to which is passed as the first parameter. 
//
// Parameters:
// -----------  
//     ppJointRef1               - The reference to a joint that this joint will mirror
//     ppJointName               - The name of this joint - This is used when position files are generated
//     ppObjectRef1              - The reference number of a body part that has previously been added to the array of body parts
//     ppObjectRef2              - The reference number of a second body part that has previously been added to the array of body parts

#macro ppMirrorJointDefinition(ppJointRef1, ppJointName,ppObjectRef1, ppObjectRef2) 
  #declare ppJointNames[ppJointCount] = ppJointName;
  #declare ppJointObjects[ppJointCount][0] = ppObjectRef1;
  #declare ppJointObjects[ppJointCount][1] = ppObjectRef2;
  #declare ppJointCentres[ppJointCount][0] = ppJointCentres[ppJointRef1][0]*<-1,1,1>;
  #declare ppJointCentres[ppJointCount][1] = ppJointCentres[ppJointRef1][1]*<-1,1,1>;
  // Mirror the y and z axes 
  #local ppI = 0;
  #while (ppI<3)
    #if (ppJointAxes[ppJointRef1][ppI].x=1 | ppJointAxes[ppJointRef1][ppI].x=-1)
      #declare ppJointAxes[ppJointCount][ppI] = ppJointAxes[ppJointRef1][ppI];
    #else
      #declare ppJointAxes[ppJointCount][ppI] = -ppJointAxes[ppJointRef1][ppI];
    #end 
    #local ppI = ppI + 1;
  #end 
  #declare ppJointLimits[ppJointCount][0][0] =ppJointLimits[ppJointRef1][0][0];
  #declare ppJointLimits[ppJointCount][0][1] =ppJointLimits[ppJointRef1][0][1];
  #declare ppJointLimits[ppJointCount][1][0] =ppJointLimits[ppJointRef1][1][0];
  #declare ppJointLimits[ppJointCount][1][1] =ppJointLimits[ppJointRef1][1][1];
  #declare ppJointLimits[ppJointCount][2][0] =ppJointLimits[ppJointRef1][2][0];
  #declare ppJointLimits[ppJointCount][2][1] =ppJointLimits[ppJointRef1][2][1];
  #declare ppJointRotations[ppJointCount][0] = 0;
  #declare ppJointRotations[ppJointCount][1] = 0; 
  #declare ppJointRotations[ppJointCount][2] = 0; 
  ppJointCount;    // Return a reference to the joint just created   
  #declare ppJointCount = ppJointCount+1; 
#end


// This macro calculates the current position of a joint based upon the position and orientation of the first of the two objects 
// that the joint connects together.
//
// Parameters:
// -----------
//     ppJointRef    - The reference number of a joint that has previously been added to the array of joints

#macro ppJointLocation (ppJointRef)
  #local ppBodyPartRef = ppJointObjects[ppJointRef][0];
  #local ppJointOffSet = ppJointCentres[ppJointRef][0]*ppBodyPartListScale[ppBodyPartRef];
  #local ppJointOffSet = vrotate(ppJointOffSet,ppBodyPartListOrientation[ppBodyPartRef]);
  #local ppBodyPartOrigin = ppBodyPartListPosition[ppBodyPartRef];   
  #local ppJointPosition = ppBodyPartOrigin+ppJointOffSet;
  ppJointPosition;
#end


// This macro returns the rotation angle settings of a joint. 
//
// Parameters:
// -----------
//     ppJointRef    - The reference number of a joint that has previously been added to the array of joints

#macro ppJointAngles (ppJointRef)
  <ppJointRotations[ppJointRef][0],
   ppJointRotations[ppJointRef][1],
   ppJointRotations[ppJointRef][2]>
#end

// This macro is used to pose body parts by defining the rotations at a joint
//
// Parameters:
// -----------
//     ppJointRef                   - The reference number of a joint that has previously been added to the array of joints
//     ppJointCentralAxisRotation   - The angle in degrees that the second object should be twisted around its own axis
//     ppJointPrimaryAxisRotation   - The angle in degrees that the second object should be rotated around the primary axis of the joint
//     ppJointSecondaryAxisRotation - The angle in degrees that the second object should be rotated around the secondary axis of the joint

#macro ppRotateJoint (ppJointRef,ppJointCentralAxisRotation,ppJointPrimaryAxisRotation,ppJointSecondaryAxisRotation)
  #declare ppJointRotations[ppJointRef][0] = ppJointCentralAxisRotation;
  #declare ppJointRotations[ppJointRef][1] = ppJointPrimaryAxisRotation; 
  #declare ppJointRotations[ppJointRef][2] = ppJointSecondaryAxisRotation; 
#end 


// This macro takes a copy of all of the joint positions. This is primarily for animations so that a second set of joint positions can be 
// loaded and an intermediary position calculated using the ppInterpolateJoint() macro.
//
// Parameters:
// -----------
// - This macro has no parameters

#macro ppStashJointRotations() 
  #local ppI = 0;
  #while (ppI<ppJointCount)
    #local ppJ = 0;
    #while (ppJ<3)
      #declare ppJointRotationStash[ppI][ppJ] = ppJointRotations[ppI][ppJ];
      #declare ppJointRotations[ppI][ppJ] = 0; 
      #local ppJ = ppJ + 1;
    #end
    #local ppI = ppI + 1; 
  #end
#end

// This macro reads two sets of the joint positions and calculates a third set of joint positions between the two.
// The first set is read from the ppJointRotationStash array and the second set is read from the ppJointRotations array.
// The results are written back into the ppJointRotations array, overwriting the second set. 
// This macro is primarily designed for animations so that an intermediary position between two fixed positions can be calculated 
//
// Parameters:
// -----------
// ppProportion       - A value between 0 and 1 defining how far we are through the transition from the first set of joint positions to the second set of joint positions

#macro ppInterpolatePose(ppProportion) 
  #local ppI = 0;
  #while (ppI<ppJointCount)
    #local ppJ = 0;
    #while (ppJ<3)
      #declare ppJointRotations[ppI][ppJ] = ppJointRotationStash[ppI][ppJ] + ppProportion*(ppJointRotations[ppI][ppJ]-ppJointRotationStash[ppI][ppJ]);  
      #local ppJ = ppJ + 1;
    #end
    #local ppI = ppI + 1; 
  #end
#end

// This macro uses the position and orientation of a given object and calculates the position of the rest.
// To do this it runs through the list of joints, inspecting the object on either side of the joint and looking for definitions
// of its position and orientation. If it finds the right information it calculates any missing position and orientation information
// for the other object connected by that joint definition. If it hasn't got enough information it leaves it for the time being.
// The list of joints is traversed repeatedly until it completes a run without making any changes. This could be because it's finished
// or it could be that there is some sort of a problem with the connectivity of the body parts. For example, if there is a joint 
// missing and one side of the break does not have sufficient data to complete the job.
//
// Parameters:
// -----------
// - This macro has no parameters

#macro ppCalculateBodyPartTransformations ()
  #local ppContinue = 1;
  #while (ppContinue)      // Loop through the list of joints until we've calculated all of the values possible 
    #local ppChangeMade = 0;
    #local ppI = 0;
    #while (ppI < ppJointCount)
      #local ppObj0Ref = ppJointObjects[ppI][0];
      #local ppObj1Ref = ppJointObjects[ppI][1];
//      #ifndef (ppBodyPartListScale[ppObj0Ref]) #declare ppBodyPartListScale[ppObj0Ref] = <1,1,1>; #end
//      #ifndef (ppBodyPartListScale[ppObj1Ref]) #declare ppBodyPartListScale[ppObj1Ref] = <1,1,1>; #end
      
      #ifndef (ppBodyPartListOrientation[ppObj0Ref])
        #ifdef (ppBodyPartListOrientation[ppObj1Ref]) 
          #ifdef (ppJointRotations[ppI][0])
          
            // Given the orientation of one body part and the rotations at the joint, work out the orientation of the other body part.
            // Rotate x and y vectors using 2 sets of rotations. The first 3 rotations are around arbitrary axes as defined for the 
            // joint. The second set is the orientation of the body part defined for ppObj1Ref.   

            // The first two axes of rotation for the joint are relative to the x, y and z axes of the object.
            #local ppAxis0 = ppJointAxes[ppI][0];
            #local ppAxis1 = ppJointAxes[ppI][1];
            // The last axis of rotation for the joint takes account of the second rotation.
            #local ppAxis2 = vaxis_rotate(ppJointAxes[ppI][2],ppAxis1,ppJointRotations[ppI][1]);

            #local ppx1    = vaxis_rotate(x   ,ppAxis2,-ppJointRotations[ppI][2]);
            #local ppx1    = vaxis_rotate(ppx1,ppAxis1,-ppJointRotations[ppI][1]);
            #local ppx1    = vaxis_rotate(ppx1,ppAxis0,-ppJointRotations[ppI][0]);

            #local ppy1    = vaxis_rotate(y   ,ppAxis2,-ppJointRotations[ppI][2]);
            #local ppy1    = vaxis_rotate(ppy1,ppAxis1,-ppJointRotations[ppI][1]);
            #local ppy1    = vaxis_rotate(ppy1,ppAxis0,-ppJointRotations[ppI][0]);
              
              
            // Rotate the reoriented x and y vectors to take account of the known orientation of the 
            // body part to which this body part is attached.
            #local ppx1 = vrotate(ppx1,ppBodyPartListOrientation[ppObj1Ref]);
            #local ppy1 = vrotate(ppy1,ppBodyPartListOrientation[ppObj1Ref]);

            // Work out a single set of rotations that will orient the body part as required
            #local ppOrientZ = VRotationD(x,<ppx1.x,ppx1.y,0>,z);                           
            #local ppOrientY = VRotationD(x,<vlength(<ppx1.x,ppx1.y,0>),0,ppx1.z>,y);    
            #local ppOrientX = VRotationD(y,vrotate(vrotate(ppy1,<0,0,-ppOrientZ>),<0,-ppOrientY,0>),x);

            // Record the newly calculated set of rotations for this body part in the right element of an array
            #declare ppBodyPartListOrientation[ppObj0Ref] = <ppOrientX,ppOrientY,ppOrientZ>;

/*
-----------------------

            #local ppx1 = vaxis_rotate(vaxis_rotate(vaxis_rotate(x,ppJointAxes[ppI][2],-ppJointRotations[ppI][2]),ppJointAxes[ppI][1],-ppJointRotations[ppI][1]),ppJointAxes[ppI][0],-ppJointRotations[ppI][0]);
            #local ppx1 = vrotate(ppx1,ppBodyPartListOrientation[ppObj1Ref]);

            #local ppy1 = vaxis_rotate(vaxis_rotate(vaxis_rotate(y,ppJointAxes[ppI][2],-ppJointRotations[ppI][2]),ppJointAxes[ppI][1],-ppJointRotations[ppI][1]),ppJointAxes[ppI][0],-ppJointRotations[ppI][0]);
            #local ppy1 = vrotate(ppy1,ppBodyPartListOrientation[ppObj1Ref]);
               
            #local ppOrientZ = VAngleD(x,<ppx1.x,ppx1.y,0>);
            #local ppOrientY = VAngleD(x,<vlength(<ppx1.x,ppx1.y,0>),0,ppx1.z>);

            #local ppOrientX = VAngleD(y,vrotate(vrotate(ppy1,<0,0,ppOrientZ>),<ppOrientY,0,0>));

            #declare ppBodyPartListOrientation[ppObj0Ref] = <ppOrientX,ppOrientY,ppOrientZ>;
*/
//#debug concat("ppObj1->ppObj0 ",vstr(3,ppBodyPartListOrientation[Obj2Ref],",",0,1),"\n") 

            #local ppChangeMade = 1;
          #end
        #end
      #end
      #ifndef (ppBodyPartListPosition[ppObj0Ref])
        #ifdef (ppBodyPartListOrientation[ppObj1Ref]) 
          #ifdef (ppBodyPartListPosition[ppObj1Ref]) 
            #declare ppBodyPartListPosition[ppObj0Ref] = 
              ppBodyPartListPosition[ppObj1Ref] +
              vrotate( ppJointCentres[ppI][1],ppBodyPartListOrientation[ppObj1Ref])*ppBodyPartListScale[ppObj1Ref] +
              vrotate(-ppJointCentres[ppI][0],ppBodyPartListOrientation[ppObj0Ref])*ppBodyPartListScale[ppObj0Ref];
            #local ppChangeMade = 1;
          #end
        #end
      #end
      #ifndef (ppBodyPartListOrientation[ppObj1Ref])
        #ifdef (ppBodyPartListOrientation[ppObj0Ref])
          #ifdef (ppJointRotations[ppI][0])
          
//#debug concat("ppJointRotations ",vstr(3,ppJointRotations[ppI][0],",",0,1),"\n") 
            // Rotate x and y vectors using 2 sets of rotations. The first 3 rotations are around arbitrary axes as defined for the 
            // joint. The second set is the orientation of the body part defined for ppObj0Ref.   

            // The first two axes of rotation for the joint are relative to the x, y and z axes of the object.
            #local ppAxis0 = ppJointAxes[ppI][0];
            #local ppAxis1 = ppJointAxes[ppI][1];
            // The last axis of rotation for the joint takes account of the second rotation.
            #local ppAxis2 = vaxis_rotate(ppJointAxes[ppI][2],ppAxis1,ppJointRotations[ppI][1]);

//#debug concat("ppAxis0 ",vstr(3,ppAxis0,",",0,1),"\n") 
//#debug concat("ppAxis1 ",vstr(3,ppAxis1,",",0,1),"\n") 
//#debug concat("ppAxis2 ",vstr(3,ppAxis2,",",0,1),"\n") 

            #local ppx1    = vaxis_rotate(x,ppAxis0,ppJointRotations[ppI][0]);
            #local ppx1    = vaxis_rotate(ppx1,ppAxis1,ppJointRotations[ppI][1]);
            #local ppx1    = vaxis_rotate(ppx1,ppAxis2,ppJointRotations[ppI][2]);

            #local ppy1    = vaxis_rotate(y,ppAxis0,ppJointRotations[ppI][0]);
            #local ppy1    = vaxis_rotate(ppy1,ppAxis1,ppJointRotations[ppI][1]);
            #local ppy1    = vaxis_rotate(ppy1,ppAxis2,ppJointRotations[ppI][2]);
              
              
            // Rotate the reoriented x and y vectors to take account of the known orientation of the 
            // body part to which this body part is attached.
            #local ppx1 = vrotate(ppx1,ppBodyPartListOrientation[ppObj0Ref]);
            #local ppy1 = vrotate(ppy1,ppBodyPartListOrientation[ppObj0Ref]);

//#debug concat("ppx1 ",vstr(3,ppx1,",",0,1),"\n") 
//#debug concat("ppx1 ",vstr(3,ppx1,",",0,1),"\n") 
//#debug concat("ppy1 ",vstr(3,ppy1,",",0,1),"\n") 
//#debug concat("ppy1 ",vstr(3,ppy1,",",0,1),"\n") 
               
            // Work out a single set of rotations that will orient the body part as required
            #local ppOrientZ = VRotationD(x,<ppx1.x,ppx1.y,0>,z);                           
            #local ppOrientY = VRotationD(x,<vlength(<ppx1.x,ppx1.y,0>),0,ppx1.z>,y);    
            #local ppOrientX = VRotationD(y,vrotate(vrotate(ppy1,<0,0,-ppOrientZ>),<0,-ppOrientY,0>),x);

//#debug concat("ppOrientX ",vstr(3,ppOrientX,",",0,1),"\n") 
//#debug concat("ppOrientY ",vstr(3,ppOrientY,",",0,1),"\n") 
//#debug concat("ppOrientZ ",vstr(3,ppOrientZ,",",0,1),"\n") 

            // Record the newly calculated set of rotations for this body part in the right element of an array
            #declare ppBodyPartListOrientation[ppObj1Ref] = <ppOrientX,ppOrientY,ppOrientZ>;
//#debug concat("ppObj0->ppObj1 ",vstr(3,ppBodyPartListOrientation[Obj2Ref],",",0,1),"\n") 


            #local ppChangeMade = 1;
          #end
        #end
      #end
      #ifndef (ppBodyPartListPosition[ppObj1Ref])
        #ifdef (ppBodyPartListOrientation[ppObj0Ref]) 
          #ifdef (ppBodyPartListPosition[ppObj0Ref]) 
            #declare ppBodyPartListPosition[ppObj1Ref] = 
              ppBodyPartListPosition[ppObj0Ref] +
              vrotate( ppJointCentres[ppI][0],ppBodyPartListOrientation[ppObj0Ref])*ppBodyPartListScale[ppObj0Ref] +
              vrotate(-ppJointCentres[ppI][1],ppBodyPartListOrientation[ppObj1Ref])*ppBodyPartListScale[ppObj1Ref];
            #local ppChangeMade = 1;
          #end
        #end
      #end    
      #local ppI = ppI + 1;
    #end
    #if (ppChangeMade=0) #local ppContinue = 0; #end
  #end  
#end

// This macro reads the list of body parts, their scale factors, their positions and their orientations from the main POV-Person
// control arrays and builds a union of those object, translating, rotating and scaling them as necessary. The object returned from 
// this macro can be embedded directly into a scene file or assigned to an object definition for subsequent use by the scene file.
// 
// Parameters:
// -----------
// - This macro has no parameters

#macro ppBuildFigure ()
  union {
    #local ppI = 0;
    #while (ppI < ppBodyPartCount) 
      object {ppBodyPartListObject[ppI] 
        scale     ppBodyPartListScale[ppI] 
        #ifdef(ppBodyPartListOrientation[ppI]) rotate    ppBodyPartListOrientation[ppI]  #end
        #ifdef(ppBodyPartListPosition[ppI])    translate ppBodyPartListPosition[ppI]     #end
        }
      #local ppI = ppI + 1;
    #end 
  } 
#end

// This scale draws lines at 0.1 unit intervals to help see the dimensions of the anatomical construct being worked on.
#declare ppAnatomyScale = union {
  cylinder {<0.5,0,0><0.5,2  ,0>,0.006  pigment {color rgb <8,8,8>}}
  
  cylinder {< 0.5,2   ,0>< 0.5,2.1 ,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {< 0.4,1.95,0>< 0.4,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {< 0.3,1.95,0>< 0.3,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {< 0.2,1.95,0>< 0.2,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {< 0.1,1.95,0>< 0.1,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {< 0  ,1.8 ,0>< 0  ,2.2 ,0>,0.006  pigment {color rgb <8,8,8>}}
  cylinder {<-0.1,1.95,0><-0.1,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {<-0.2,1.95,0><-0.2,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {<-0.3,1.95,0><-0.3,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {<-0.4,1.95,0><-0.4,2.05,0>,0.005  pigment {color rgb <8,0,0>}}
  cylinder {<-0.5,1.9 ,0><-0.5,2.1 ,0>,0.005  pigment {color rgb <8,0,0>}}
  
  cylinder {<0.3 ,0  ,0><0.7 ,0  ,0>,0.006 pigment {color rgb <8,8,8>}}
  cylinder {<0.45,0.1,0><0.55,0.1,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,0.2,0><0.55,0.2,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,0.3,0><0.55,0.3,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,0.4,0><0.55,0.4,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.4 ,0.5,0><0.6 ,0.5,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,0.6,0><0.55,0.6,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,0.7,0><0.55,0.7,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,0.8,0><0.55,0.8,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,0.9,0><0.55,0.9,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.3 ,1  ,0><0.7 ,1  ,0>,0.006 pigment {color rgb <8,8,8>}}
  cylinder {<0.45,1.1,0><0.55,1.1,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,1.2,0><0.55,1.2,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,1.3,0><0.55,1.3,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,1.4,0><0.55,1.4,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.4 ,1.5,0><0.6 ,1.5,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,1.6,0><0.55,1.6,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,1.7,0><0.55,1.7,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,1.8,0><0.55,1.8,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<0.45,1.9,0><0.55,1.9,0>,0.005 pigment {color rgb <8,0,0>}}
  cylinder {<-0.5 ,2  ,0><0.7 ,2  ,0>,0.006 pigment {color rgb <8,8,8>}}
}

/****************************************/
/*  There follow some utility macros    */
/****************************************/

#declare ppTempArray = array[200][2]; 


//  This macro reads pairs of vectors from a CSV file into the first rows of a pair of arrays. 
//  The first vector of each pair represents a position along a line.
//  The second vector of each pair represents a surface normal at that point. 
//
// Parameters:
// -----------
// ppCSVFileID     - The name of the CSV file to read
// ppPositionArray - The array that we will write the set of positions into
// ppNormalArray   - The array that we will write the set of normals into
// ppSegments      - The number of points along the line to read from the file

#macro ppReadLineFromCSV (ppCSVFileID, ppPositionArray, ppNormalArray, ppSegments)
  #declare ppI = 0;
  #declare ppJ = 0;
  #fopen ppCSVFileHandle ppCSVFileID read
  #while (defined(ppCSVFileHandle))
    #read (ppCSVFileHandle,ppPosition,ppNormal)
    #declare ppPositionArray[0][ppJ] = ppPosition;
    #declare ppNormalArray  [0][ppJ] = ppNormal;
    #declare ppJ = ppJ + 1;
  #end 
  #local ppSegments = ppJ;
  #fclose ppCSVFileHandle 
#end

// This macro takes a 2D array that contains a set of positions in the first row and generates a series of positions to form a
// curved eyelid shape that is written back into the remaining rows of the array.  
//
// Parameters:
// -----------
// ppArray         - The array
// ppSlices        - The number of slices (used for the first array dimension)
// ppSegments      - The number of segments (used for the second array dimension)
// ppProportion    - A number representing how open the eye lid is
// ppUpperInd      - 1=Upper eye lid, -1=Lower eye lid
// ppLeftInd       - 1=Left eye lid,  -1=Right eye lid

#macro ppExtrudeEyeLid (ppArray, ppSlices, ppSegments, ppProportion, ppUpperInd, ppLeftInd)
  #local ppThisEyePosition = ppEyePosition*<ppLeftInd,1,1>;
  #local ppDimI = ppSlices-1;
  #local ppDimJ = ppSegments;
  #local ppJ = 0;    
  #while (ppJ<ppDimJ)
    #local ppExtentVector = VProject_Plane(ppArray[0][ppJ]-ppThisEyePosition, ppAxis);
    #local ppBaseVector   = VProject_Plane(ppExtentVector, ppBasePlaneNormal);
    #local ppLidSweep     = VAngleD(ppExtentVector,ppBaseVector);
    #local ppTotalAngle   = ppProportion*ppLidSweep;
    #local ppI = 1;
    #while (ppI<ppDimI)
      #local ppThisAngle = ppTotalAngle*ppI/(ppDimI-1);
      #declare ppArray[ppI][ppJ] = vaxis_rotate(ppArray[0][ppJ]-ppThisEyePosition,ppAxis, ppThisAngle*ppUpperInd)+ppThisEyePosition;
      #if (ppI=ppDimI-1) #declare ppArray[ppI+1][ppJ] = ppArray[ppI][ppJ]+<0,-0.5*ppUpperInd,3>; #end
      #local ppI = ppI + 1;
    #end
    #local ppJ = ppJ + 1;
  #end  
#end
       


//  This macro positions copies of an object at regular intervals along a line defined by an array of points.
//  In fact the line of points is taken from a 2D array, but the first index to the array is constant.
//  It uses a very simple linear interpolation at regular intervals in the direction of the X axis.
//  The array must therefore contain sequential points sequenced from +X to -X (or from -x to +x if ppDirection=-1).                                        
//  The object is also re-oriented at each point as defined by an array of normals.
//
// Parameters:
// -----------
// ppObject        - The object to be copied along a line
// ppArray         - A 2D array of points
// ppNormal        - A 2D array of normals at the points in ppArray 
// ppI             - The index to the first dimension of the array
// ppDimJ          - The number of points to read from the second dimension of the array
// ppObjectSpacing - How far apart to position the copies of the object
// ppDirection     - 1=Right to Left, -1=Left to Right

#macro ppLineOfEyeLashes (ppObject, ppArray, ppNormal, ppI, ppDimJ, ppObjectSpacing, ppDirection)
  #ifndef (ppObjectSpacingSeed) #declare ppObjectSpacingSeed = seed(1); #end
  #local ppJ = 0;
  #local ppXDistance = ppArray[ppI][0].x;
  #while (ppJ<ppDimJ-1)
    #local ppNextJ = ppJ + 1;
    #while (ppXDistance*ppDirection > ppArray[ppI][ppNextJ].x*ppDirection)
      #local ppRatio = (ppArray[ppI][ppJ].x - ppXDistance)/(ppArray[ppI][ppJ].x - ppArray[ppI][ppNextJ].x);
      #local ppPosition = ppArray[ppI][ppJ]+(ppArray[ppI][ppNextJ] - ppArray[ppI][ppJ])*ppRatio;
      #local ppNorm     = ppNormal[0][ppJ]+(ppNormal[0][ppNextJ] - ppNormal[0][ppJ])*ppRatio;
      object {ppObject Reorient_Trans(<-0.1+0.2*rand(ppObjectSpacingSeed),-0.1+0.2*rand(ppObjectSpacingSeed),-1>, -ppNorm) translate ppPosition}
      #local ppXDistance = ppXDistance - ppDirection*ppObjectSpacing*(1.2-0.4*rand(ppObjectSpacingSeed)); 
      #local ppObject = object {ppObject scale (0.95+0.07*rand(ppObjectSpacingSeed))}
//#debug concat("ppPosition = ",vstr(3,ppPosition ",",3,0),":","\n")
    #end
    #local ppJ = ppJ + 1;
  #end 
#end 

//  This macro generates smooth triangles that fill in a strip between an array of points and a 
//  parallel line at a given offset.
//
// Parameters:
// -----------
// ppOffset       - A 3D vector defining an offset from the points in the ppTempArray
// ppShiftArray   - If this parameter is set to "Shift Array" the ppTempArray is updated with the new points
// ppRotateNormal - A 3D vector defining the angles through which the surface normal should be rotated

#macro ppExtrudeFromArray(ppOffset,ppShiftArray,ppRotateNormal)
  #local ppI = 0;
  #while (ppI<ppTempArrayIndex-1)
    #local ppNext = ppI + 1;
    smooth_triangle { 
      ppTempArray[ppI][0]            ,        ppTempArray[ppI][1],
      ppTempArray[ppNext][0]         ,        ppTempArray[ppNext][1],
      ppTempArray[ppI][0]   +ppOffset,vrotate(ppTempArray[ppI][1]   ,ppRotateNormal)
    }
    smooth_triangle { 
      ppTempArray[ppNext][0]+ppOffset,vrotate(ppTempArray[ppNext][1],ppRotateNormal),
      ppTempArray[ppNext][0]         ,        ppTempArray[ppNext][1],
      ppTempArray[ppI][0]   +ppOffset,vrotate(ppTempArray[ppI][1]   ,ppRotateNormal)
    }
    #local ppI = ppI + 1;
  #end
  #if (strcmp(ppShiftArray,"Shift Array")=0)
    #local ppI = 0;
    #while (ppI<ppTempArrayIndex)
      #declare ppTempArray[ppI][0] = ppTempArray[ppI][0] + ppOffset;
      #declare ppTempArray[ppI][1] = vrotate(ppTempArray[ppI][1]   ,ppRotateNormal);
      #local ppI = ppI + 1;
    #end
  #end
#end


// This macro defines a set of points by extruding points contained within the first line of an Array along a path dictated 
// by a set of offsets. A rotation and a scaling are applied to the offsets to help create a lip shape that is acceptable both 
// when the mouth is open and when it is closed.
//
// Parameters:
// -----------
//     ppArray            - The 2 dimensional array already containing the first line of points
//     ppNormal           - The 2 dimensional array already containing the first line of normals
//     ppDimI             - The first dimension of the two arrays 
//     ppDimJ             - The second dimension of the two arrays 
//     ppProfile          - The one dimensional array containing the offsets
//     ppOrientation      - The direction of rotation to tip the lip profile over as the distance from the x-axis increases
//
#macro ppExtrudeLipAlongProfile(ppArray, ppNormal, ppDimI, ppDimJ, ppProfile, ppOrientation)
  #local ppJ = 0;
  #while (ppJ<ppDimJ)
    // For points further from the centre, tip the offsets over to help make the lips round when the mouth is open   
    #local ppOffsetRotation = 0.6*z*pow(ppArray[0][ppJ].x,2)*ppOrientation; 
    #local ppI = 1;
    #while (ppI<ppDimI)
      // Scale down the offset as it gets further from the centre, so the lips are fuller in the middle
      #local ppScaledOffset    = ppLipProfile[ppI-1]*(1-ppArray[0][ppJ].x/40)*<1,2,0.5>;  
      #local ppRotatedOffset   = vrotate(ppScaledOffset,ppOffsetRotation); 
      #local ppArray[ppI][ppJ] = ppArray[ppI-1][ppJ]+ppRotatedOffset;
      #local ppI = ppI + 1;
    #end
    #local ppJ = ppJ + 1;
  #end
#end


// This macro takes one or two joint references as its parameters. 
// If one joint is given, it reverses the direction of rotation in y and the direction of rotation in z that has been defined for the joint.
// Otherwise it checks the orientation of the 3 axes of each joint and, if different reverses the direction of rotation before swapping values between the two joints. 
//
// Parameters:
// -----------
//     ppJointRef1        - The first joint reference
//     ppJointRef2        - The second joint reference, or -1 if only one joint reference is to be used
//
#macro ppMirrorJoints(ppJointRef1,ppJointRef2)
  #if (ppJointRef2=-1)   // If the joint lies on the central axis, there's only one joint to mirror 
//#debug concat(str(ppJointRotations[ppJointRef1][0],3,3),";",str(ppJointRotations[ppJointRef1][2],3,3),"\n")     
    #declare ppJointRotations[ppJointRef1][0] = ppJointRotations[ppJointRef1][0]*-1;
    #declare ppJointRotations[ppJointRef1][2] = ppJointRotations[ppJointRef1][2]*-1;
  #else
    #local ppI=0;
    #while (ppI<3)
      #local ppRotation = ppJointRotations[ppJointRef1][ppI];
      #declare ppJointRotations[ppJointRef1][ppI] = ppJointRotations[ppJointRef2][ppI];
      #declare ppJointRotations[ppJointRef2][ppI] = ppRotation;
      #local ppI = ppI + 1;
    #end
  #end 
#end

// This macro stores the values of a fixed body part from which the position and orientation of all of the other body parts will be calculated.
//
// Parameters:
// -----------
//     ppBodyPartRef      - The reference number of the body part to be used as the fixed body part
//     ppPosition         - The position of that body part
//     ppOrientation      - The orientation of the fixed body part
//
#macro ppFixBodyPart(ppBodyPartRef, ppPosition, ppOrientation)
  #declare ppFixedBodyPart            = ppBodyPartRef;
  #declare ppFixedBodyPartPosition    = ppPosition;
  #declare ppFixedBodyPartOrientation = ppOrientation;
#end  



// This macro writes the current set of joint rotations to a file in a format suitable for use as a positions include file. 
// This can be useful if it is necessary to keep a record of a generated position.
//
// Parameters:
// -----------
//     ppFileID          - The ID of the file to write the definitions into 
//     ppShowAll         - 0=Only output non-default values;  1=Output everyting
#macro ppWriteJointRotationsToFile (ppFileID, ppShowAll)

  #fopen ppFileHandle ppFileID write 
  #write (ppFileHandle,concat("/***************************************************************************************************/","\n"))
  #write (ppFileHandle,concat("/* ",ppFileID,"  - Copyright Chris Bartlett September 2005                                       */","\n"))
  #write (ppFileHandle,concat("/* This file was generated using POV-Person and is licensed for royalty free re-use under the      */","\n"))
  #write (ppFileHandle,concat("/* terms of the POV-Person License.                                                                */","\n"))
  #write (ppFileHandle,concat("/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */","\n"))
  #write (ppFileHandle,concat("/***************************************************************************************************/","\n"))
  #write (ppFileHandle,concat("","\n"))
  
  #local ppI = 0;
  #while (ppI<ppJointCount)
    #if (ppShowAll | ppJointRotations[ppI][0]!=0 | ppJointRotations[ppI][1]!=0 | ppJointRotations[ppI][2]!=0) 
      #write (ppFileHandle,concat("ppRotateJoint(",ppJointNames[ppI],",",str(ppJointRotations[ppI][0],3,3),",",str(ppJointRotations[ppI][1],3,3),",",str(ppJointRotations[ppI][2],3,3),")","\n"))
    #end
    #local ppI = ppI + 1;
  #end
//  #fclose ppFileHandle

#end    


#macro ppAdjustEyes()
  #ifdef (ppTransformFigure)
    #local ppDirectionVector = vinv_transform(ppLookAt, ppTransformFigure) - ppBodyPartListPosition[ppLeftEyeRef];
  #else
    #local ppDirectionVector = ppLookAt - ppBodyPartListPosition[ppLeftEyeRef];
  #end   
  #declare ppBodyPartListObject[ppLeftEyeRef ] = object {ppBodyPartListObject[ppLeftEyeRef ] Reorient_Trans(z,ppDirectionVector)}

//#debug concat("Left Eye",vstr(3,ppDirectionVector,",",3,3),"\n")
  #ifdef (ppTransformFigure)
    #local ppDirectionVector = vinv_transform(ppLookAt, ppTransformFigure) - ppBodyPartListPosition[ppRightEyeRef];
  #else
    #local ppDirectionVector = ppLookAt - ppBodyPartListPosition[ppRightEyeRef];
  #end   
  #declare ppBodyPartListObject[ppRightEyeRef] = object {ppBodyPartListObject[ppRightEyeRef] Reorient_Trans(z,ppDirectionVector)}
#end

