/***************************************************************************************************/
/* ppf1scaling.inc  - Copyright Chris Bartlett 2005                                                */
/* This work is licensed for royalty free re-use under the terms of the POV-Person License.        */
/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */
/***************************************************************************************************/
  
// This macro is used to add a series of scale factors to the parts of a figure. This macro is specific to a particular type of figure (e.g. "f1"). 
//
// Parameters:
// -----------  
// - This macro has no parameters
// 
#macro ppScaleBodyParts()
  ppScaleBodyPart (ppLeftThumbBaseRef          ,<1,1,1>);
  ppScaleBodyPart (ppLeftThumbMiddleRef        ,<1,1,1>);
  ppScaleBodyPart (ppLeftThumbTipRef           ,<1.2,1  ,1.2>);
  ppScaleBodyPart (ppLeftThumbNailRef          ,<1.2,1  ,1.2>); 
  ppScaleBodyPart (ppLeftFirstFingerBaseRef    ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppLeftFirstFingerMiddleRef  ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppLeftFirstFingerTipRef     ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppLeftFirstFingerNailRef    ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppLeftMiddleFingerBaseRef   ,<1,1,1>);            
  ppScaleBodyPart (ppLeftMiddleFingerMiddleRef ,<1,1,1>);                   
  ppScaleBodyPart (ppLeftMiddleFingerTipRef    ,<1,1,1>);
  ppScaleBodyPart (ppLeftMiddleFingerNailRef   ,<1,1,1>);
  ppScaleBodyPart (ppLeftThirdFingerBaseRef    ,<1,1,1>);            
  ppScaleBodyPart (ppLeftThirdFingerMiddleRef  ,<1,1,1>);                   
  ppScaleBodyPart (ppLeftThirdFingerTipRef     ,<1,1,1>);
  ppScaleBodyPart (ppLeftThirdFingerNailRef    ,<1,1,1>);
  ppScaleBodyPart (ppLeftLittleFingerBaseRef   ,<0.8,0.8,0.8>);
  ppScaleBodyPart (ppLeftLittleFingerMiddleRef ,<0.8,0.8,0.8>);        
  ppScaleBodyPart (ppLeftLittleFingerTipRef    ,<0.8,0.8,0.8>);      
  ppScaleBodyPart (ppLeftLittleFingerNailRef   ,<0.8,0.8,0.8>);      
  
  ppScaleBodyPart (ppRightThumbBaseRef          ,<1,1,1>);
  ppScaleBodyPart (ppRightThumbMiddleRef        ,<1,1,1>);
  ppScaleBodyPart (ppRightThumbTipRef           ,<1.2,1  ,1.2>);
  ppScaleBodyPart (ppRightThumbNailRef          ,<1.2,1  ,1.2>); 
  ppScaleBodyPart (ppRightFirstFingerBaseRef    ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppRightFirstFingerMiddleRef  ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppRightFirstFingerTipRef     ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppRightFirstFingerNailRef    ,<0.95,0.95,0.95>);
  ppScaleBodyPart (ppRightMiddleFingerBaseRef   ,<1,1,1>);            
  ppScaleBodyPart (ppRightMiddleFingerMiddleRef ,<1,1,1>);                   
  ppScaleBodyPart (ppRightMiddleFingerTipRef    ,<1,1,1>);
  ppScaleBodyPart (ppRightMiddleFingerNailRef   ,<1,1,1>);
  ppScaleBodyPart (ppRightThirdFingerBaseRef    ,<1,1,1>);            
  ppScaleBodyPart (ppRightThirdFingerMiddleRef  ,<1,1,1>);                   
  ppScaleBodyPart (ppRightThirdFingerTipRef     ,<1,1,1>);
  ppScaleBodyPart (ppRightThirdFingerNailRef    ,<1,1,1>);
  ppScaleBodyPart (ppRightLittleFingerBaseRef   ,<0.8,0.8,0.8>);
  ppScaleBodyPart (ppRightLittleFingerMiddleRef ,<0.8,0.8,0.8>);        
  ppScaleBodyPart (ppRightLittleFingerTipRef    ,<0.8,0.8,0.8>);      
  ppScaleBodyPart (ppRightLittleFingerNailRef   ,<0.8,0.8,0.8>);      
  
  
  #local ppFootSize = <0.85,0.85,0.85>;
  
  ppScaleBodyPart (ppLeftFootRef      ,ppFootSize);  
  ppScaleBodyPart (ppRightFootRef     ,ppFootSize);  
  
  ppScaleBodyPart (ppLeftBigToeTipRef      ,<1.5,1.5,1.5>*ppFootSize);  
  ppScaleBodyPart (ppLeftBigToeNailRef     ,<1.5,1.5,1.5>*ppFootSize);
  ppScaleBodyPart (ppLeftFirstToeTipRef    ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftFirstToeNailRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftMiddleToeTipRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftMiddleToeNailRef  ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftThirdToeTipRef    ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftThirdToeNailRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftLittleToeTipRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftLittleToeNailRef  ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftFirstToeMiddleRef ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftMiddleToeMiddleRef,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftThirdToeMiddleRef ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftLittleToeMiddleRef,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftBigToeBaseRef     ,<1.4,1.4,1.5>*ppFootSize);
  ppScaleBodyPart (ppLeftFirstToeBaseRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftMiddleToeBaseRef  ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftThirdToeBaseRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppLeftLittleToeBaseRef  ,<0.9,0.9,0.9>*ppFootSize);
  
  ppScaleBodyPart (ppRightBigToeTipRef      ,<1.5,1.5,1.5>*ppFootSize);  
  ppScaleBodyPart (ppRightBigToeNailRef     ,<1.5,1.5,1.5>*ppFootSize);
  ppScaleBodyPart (ppRightFirstToeTipRef    ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightFirstToeNailRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightMiddleToeTipRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightMiddleToeNailRef  ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightThirdToeTipRef    ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightThirdToeNailRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightLittleToeTipRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightLittleToeNailRef  ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightFirstToeMiddleRef ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightMiddleToeMiddleRef,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightThirdToeMiddleRef ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightLittleToeMiddleRef,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightBigToeBaseRef     ,<1.4,1.4,1.5>*ppFootSize);
  ppScaleBodyPart (ppRightFirstToeBaseRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightMiddleToeBaseRef  ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightThirdToeBaseRef   ,<1,1,1>*ppFootSize);
  ppScaleBodyPart (ppRightLittleToeBaseRef  ,<0.9,0.9,0.9>*ppFootSize);
#end 

