/***************************************************************************************************/
/* ppuppereyelid.inc  - Copyright Chris Bartlett 16 Apr 2005                                       */
/* This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike License.  */
/* To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/1.0/ or      */
/* send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.      */
/***************************************************************************************************/ 

#ifndef(ppCamera)   #declare ppCamera="UpperLid";   #end 
#ifndef(POVPerson)  #include "povperson.inc"        #end
#ifndef(ppLimbScan) #include "ppclothingmacros.inc" #end
#include "math.inc"

#ifndef (ppVerbose)    #local ppVerbose    = 1; #end
#ifndef (ppShowPoints) #local ppShowPoints = 0; #end

#ifndef (ppLeftUpperEyelidAngle ) #declare ppLeftUpperEyelidAngle  =  10; #end
#ifndef (ppRightUpperEyelidAngle) #declare ppRightUpperEyelidAngle =  10; #end
#ifndef (ppLeftLowerEyelidAngle ) #declare ppLeftLowerEyelidAngle  = -15; #end
#ifndef (ppRightLowerEyelidAngle) #declare ppRightLowerEyelidAngle = -15; #end 

#ifndef (ppEyePosition) #declare ppEyePosition = <13.2, 84.8, -2.4>; #end                 //<13.2, 80.6, -3.5>
#ifndef (ppEyeRadius  ) #declare ppEyeRadius   = 6.2;                #end
 

//  Imagine the eye lid sliced by cuts aligned to the edge, then being divided into longitudinal segments.
#local ppUpperLidSlices   = 10;
#local ppLowerLidSlices   = 7;
#declare ppUpperLidLashLine = ppUpperLidSlices - 2;
#declare ppLowerLidLashLine = ppLowerLidSlices - 2;
#local ppMaxLidSegments   = 30;
#declare ppUpperLidSegments = ppMaxLidSegments;   // Will be adjusted when we read the file and find out how many there really are.
#declare ppLowerLidSegments = ppMaxLidSegments;   // Will be adjusted when we read the file and find out how many there really are.
#local ppkDontJoinEnds = 0;

#declare ppLeftUpperLidArray   = array[ppUpperLidSlices][ppMaxLidSegments]; 
#declare ppLeftUpperLidNormal  = array[ppUpperLidSlices][ppMaxLidSegments]; 
#declare ppLeftLowerLidArray   = array[ppLowerLidSlices][ppMaxLidSegments]; 
#declare ppLeftLowerLidNormal  = array[ppLowerLidSlices][ppMaxLidSegments]; 
#declare ppRightUpperLidArray  = array[ppUpperLidSlices][ppMaxLidSegments]; 
#declare ppRightUpperLidNormal = array[ppUpperLidSlices][ppMaxLidSegments]; 
#declare ppRightLowerLidArray  = array[ppLowerLidSlices][ppMaxLidSegments]; 
#declare ppRightLowerLidNormal = array[ppLowerLidSlices][ppMaxLidSegments];  

#if (ppVerbose = 1) #debug concat("Start of Left Eyelid","\n")  #end 
ppReadLineFromCSV ("ppf2uppereyeline.csv", ppLeftUpperLidArray, ppLeftUpperLidNormal, ppUpperLidSegments) 
ppReadLineFromCSV ("ppf2lowereyeline.csv", ppLeftLowerLidArray, ppLeftLowerLidNormal, ppLowerLidSegments)

#local ppJ = 0;
#while (ppJ<ppUpperLidSegments)
  #declare ppRightUpperLidArray [0][ppJ] = ppLeftUpperLidArray [0][ppJ]*<-1,1,1>; 
  #declare ppRightUpperLidNormal[0][ppJ] = ppLeftUpperLidNormal[0][ppJ]*<-1,1,1>;
  #local ppJ = ppJ + 1; 
#end

#local ppJ = 0;
#while (ppJ<ppLowerLidSegments)
  #declare ppRightLowerLidArray [0][ppJ] = ppLeftLowerLidArray [0][ppJ]*<-1,1,1>; 
  #declare ppRightLowerLidNormal[0][ppJ] = ppLeftLowerLidNormal[0][ppJ]*<-1,1,1>;
  #local ppJ = ppJ + 1; 
#end

// Left Eye Lid
// Work out the left and right most extents of the eyelid and define a vector joining the two points
#local ppExtent1 = ppLeftUpperLidArray[0][0];
#local ppExtent2 = ppLeftUpperLidArray[0][ppUpperLidSegments-1];
#local ppAxis = ppExtent2 - ppExtent1;
//#local ppBasePlaneNormal = VPerp_To_Plane(ppExtent1-ppEyePosition,ppExtent2-ppEyePosition);
#local ppBasePlaneNormal = y;

#local ppUpperExtentVector = VProject_Plane(ppLeftUpperLidArray[0][div(ppUpperLidSegments,2)]-ppEyePosition, ppAxis);
#local ppUpperBaseVector   = VProject_Plane(ppUpperExtentVector, ppBasePlaneNormal);
#local ppUpperLidSweep     =  VAngleD(ppUpperExtentVector,ppUpperBaseVector); 

#local ppLowerExtentVector = VProject_Plane(ppLeftLowerLidArray[0][div(ppUpperLidSegments,2)]-ppEyePosition, ppAxis);
#local ppLowerBaseVector   = VProject_Plane(ppLowerExtentVector, ppBasePlaneNormal);
#local ppLowerLidSweep     = -VAngleD(ppLowerExtentVector,ppLowerBaseVector);

// Set limits for the eye lid angles based upon the available hole size and making sure the upper and lower lid don't overlap 
#declare ppLeftUpperEyelidAngle = max(min(ppLeftUpperEyelidAngle,ppUpperLidSweep),ppLowerLidSweep);
#declare ppLeftLowerEyelidAngle = max(min(ppLeftLowerEyelidAngle,ppUpperLidSweep),ppLowerLidSweep);
#declare ppLeftLowerEyelidAngle = min(ppLeftLowerEyelidAngle,ppLeftUpperEyelidAngle);

#local ppUpperEyelidProportion = (ppUpperLidSweep-ppLeftUpperEyelidAngle)/ppUpperLidSweep;
#local ppLowerEyelidProportion = (ppLowerLidSweep-ppLeftLowerEyelidAngle)/ppLowerLidSweep;

ppExtrudeEyeLid (ppLeftUpperLidArray, ppUpperLidSlices, ppUpperLidSegments, ppUpperEyelidProportion, 1, 1)
ppExtrudeEyeLid (ppLeftLowerLidArray, ppLowerLidSlices, ppLowerLidSegments, ppLowerEyelidProportion,-1, 1)


// Right Eye Lid
// Work out the left and right most extents of the eyelid and define a vector joining the two points
#local ppExtent1 = ppRightUpperLidArray[0][0];
#local ppExtent2 = ppRightUpperLidArray[0][ppUpperLidSegments-1];
#local ppAxis = -(ppExtent2 - ppExtent1);
#local ppBasePlaneNormal = y;

#local ppUpperExtentVector = VProject_Plane(ppRightUpperLidArray[0][div(ppUpperLidSegments,2)]-ppEyePosition, ppAxis);
#local ppUpperBaseVector   = VProject_Plane(ppUpperExtentVector, ppBasePlaneNormal);
#local ppUpperLidSweep     =  VAngleD(ppUpperExtentVector,ppUpperBaseVector); 

#local ppLowerExtentVector = VProject_Plane(ppRightLowerLidArray[0][div(ppUpperLidSegments,2)]-ppEyePosition, ppAxis);
#local ppLowerBaseVector   = VProject_Plane(ppLowerExtentVector, ppBasePlaneNormal);
#local ppLowerLidSweep     = -VAngleD(ppLowerExtentVector,ppLowerBaseVector);

// Set limits for the eye lid angles based upon the available hole size and making sure the upper and lower lid don't overlap 
#declare ppRightUpperEyelidAngle = max(min(ppRightUpperEyelidAngle,ppUpperLidSweep),ppLowerLidSweep);
#declare ppRightLowerEyelidAngle = max(min(ppRightLowerEyelidAngle,ppUpperLidSweep),ppLowerLidSweep);
#declare ppRightLowerEyelidAngle = min(ppRightLowerEyelidAngle,ppRightUpperEyelidAngle);

#local ppUpperEyelidProportion = (ppUpperLidSweep-ppRightUpperEyelidAngle)/ppUpperLidSweep;
#local ppLowerEyelidProportion = (ppLowerLidSweep-ppRightLowerEyelidAngle)/ppLowerLidSweep;

ppExtrudeEyeLid (ppRightUpperLidArray, ppUpperLidSlices, ppUpperLidSegments, ppUpperEyelidProportion, 1, 1)
ppExtrudeEyeLid (ppRightLowerLidArray, ppLowerLidSlices, ppLowerLidSegments, ppLowerEyelidProportion,-1, 1)

#local ppLidOffset = <0,-0.5,3>;
#local ppEmptyArray = array[100][100]; 

#declare ppLeftUpperEyeLid1 = ppMesh2FromArray(ppLeftUpperLidArray,ppEmptyArray,ppUpperLidSlices,ppUpperLidSegments,ppkDontJoinEnds);
#declare ppLeftLowerEyeLid1 = ppMesh2FromArray(ppLeftLowerLidArray,ppEmptyArray,ppLowerLidSlices,ppLowerLidSegments,ppkDontJoinEnds);

#declare ppRightUpperEyeLid1 = ppMesh2FromArray(ppRightUpperLidArray,ppEmptyArray,ppUpperLidSlices,ppUpperLidSegments,ppkDontJoinEnds);
#declare ppRightLowerEyeLid1 = ppMesh2FromArray(ppRightLowerLidArray,ppEmptyArray,ppLowerLidSlices,ppLowerLidSegments,ppkDontJoinEnds);


#if (ppShowPoints=1) ppDrawPoints(ppLeftUpperLidArray ,ppUpperLidSlices,ppUpperLidSegments,0.1,<1,2,1>) #end 
#if (ppShowPoints=1) ppDrawPoints(ppLeftLowerLidArray ,ppLowerLidSlices,ppLowerLidSegments,0.1,<2,0,0>) #end 
#if (ppShowPoints=1) ppDrawPoints(ppRightUpperLidArray,ppUpperLidSlices,ppUpperLidSegments,0.1,<1,2,1>) #end 
#if (ppShowPoints=1) ppDrawPoints(ppRightLowerLidArray,ppLowerLidSlices,ppLowerLidSegments,0.1,<2,0,0>) #end 
                          
//#ifdef(ppFace)
//  #declare ppFace = union {
//    object {ppFace}                      
//    object {ppLeftUpperEyeLid1}
//    object {ppLeftLowerEyeLid1}
//  }
//#end

#if (strcmp(ppCamera,"UpperLid")=0)
  camera {
    location ppEyePosition+<50, 0, -40>
    look_at ppEyePosition*<-1,1,1>
    angle 34.3775
  }

  #declare ppEyeLashTexture = texture {pigment {color rgb 5}}
  #include "ppf2eyelash.inc"
  
  light_source { <250, 60,-8900> color rgb<1, 1, 1>*0.6 }   
  light_source { <0, 1400,-1600> color rgb<1, 1, 1>*1.2 }   
  light_source { <0, 60,0> color rgb<1, 0.5, 0.5>} 
   
  object {ppLeftUpperEyeLid1 pigment {color rgb <1,1,0>}}    
  object {ppLeftLowerEyeLid1 pigment {color rgb <1,1,0>}}    
  object {ppLeftUpperLash1}
  object {ppLeftLowerLash1}


  object {ppRightUpperEyeLid1 pigment {color rgb <1,1,0>}}    
  object {ppRightLowerEyeLid1 pigment {color rgb <1,1,0>}}    
  object {ppRightUpperLash1 pigment {color rgb 5}}
  object {ppRightLowerLash1 pigment {color rgb 5}}

//  #if (1)                             // 1=Show EyeBall  0=Don't Show EyeBall
//    #include "ppeye.inc"
//    object {ppEye1  translate ppEyePosition }
//  #end  
#end                                                    
