// ISSUES
// Heel orientation is not always correct, depending on the angle of the boot.
// Toe of boot needs defining for sole
// ppAddOffSetToArray probably won't animate correctly

#include "math.inc"
#include "ppclothingmacros.inc" 

// This macro generates a single boot by wrapping material around a series of objects representing elements of a leg.
//
// Parameters:
// -----------
//               - No parameters are passed to this macro. 
//
#macro ppBobbleHatGenerator ()
  #ifndef(ppVerbose)    #local ppVerbose = 1; #end
  #ifndef(ppShowPoints) #local ppShowPoints = 0; #end 
  #ifndef(ppHatTexture) #declare ppHatTexture = texture{ pigment {color rgb <1,0,0>}} #end 
  
  #local ppSegments          = 30;
  #local ppSkullSlices       = 5;
  #local ppFringeSlices      = 30;
  #local ppTotalHatSlices    = 25;
  #local ppRotationDirection = 1;                                      
  #local ppJoinEnds          = 1;
  #local ppDontJoinEnds      = 0; 
  #local ppTraceLength       = 1;
  #local ppApproxDeltaAngle  = 10;
  #local ppDeltaDistance     = 0.001;

  #local ppHatArray = array [ppTotalHatSlices][ppSegments];
  #local ppHatNormal= array [ppTotalHatSlices][ppSegments]; 

  #local ppFringeArray = array [ppFringeSlices][ppSegments];
  #local ppFringeNormal= array [ppFringeSlices][ppSegments]; 

  #local ppSkullEndVector = ppSkullStartVector;

  // Fire a ray towards the back of the skull to find a point that we can use as the end of our path through it
  #local ppNorm = <0,0,0>;  
  #declare ppSkullEndPos = trace ( ppSkullObject, ppSkullStartPos+ppSkullStartVector, -ppSkullStartVector*0.1, ppNorm);
  #if (vlength(ppNorm)!=0) 
  #else
    #warning concat("The ppBobbleHatGenerator Macro sent out a ray from point <",vstr(3,ppSkullStartPos,",",0,3),"> that didn't intercept the ppSkull object.\n")
  #end  
  #local ppEmptyArray = array[100][100]; 

  #if (ppVerbose = 1)
    #debug concat("SkullStartPos: "   ,vstr(3,ppSkullStartPos,",",3,3),"\n")
    #debug concat("SkullStartVector: ",vstr(3,ppSkullStartVector,",",3,3),"\n")
    #debug concat("SkullEndPos: "     ,vstr(3,ppSkullEndPos,",",3,3),"\n")
    #debug concat("SkullEndVector: "  ,vstr(3,ppSkullEndVector,",",3,3),"\n")
  #end

  // Scan the skull to define the first part of the hat shape
  #if (ppVerbose = 1) #debug concat("Start of Skull part of Hat","\n")  #end 
  #local ppEndRefVector   = ppStartRefVector;
  ppLimbScan(ppSkullObject,ppSkullStartPos,ppSkullStartVector,ppStartRefVector,ppSkullEndPos,ppSkullEndVector,ppEndRefVector,ppSkullSlices,ppSegments,ppHatArray,ppHatNormal,0,ppRotationDirection,ppTraceLength)
//  ppAddOffSetToArray(0, ppAbdomenSlices-1, 0, ppSegments-1, ppAbdomenArray, ppAbdomenNormal, ppFabricDisplacementPigment,0,ppFabricOffSet)

  // Work out the angles required to transition the droopy bit into the vertical. 
  #if (ppEndRefVector.y<0)
    #debug concat("Warning. The bobble hat generator only works properly when the head is the right way up.")
  #end
  #local ppTotalTurnAngle     = VAngleD(ppSkullEndVector,-y);   
  #local ppTurnSlices         = div(ppTotalTurnAngle,ppApproxDeltaAngle);
  #local ppDeltaAngle         = ppTotalTurnAngle/ppTurnSlices;
  #local ppHatSlices          = ppSkullSlices;
  #local ppRotationAxisVector = VPerp_To_Plane(ppSkullEndVector, -y); 
  #local ppHatVector          = ppSkullEndVector;
  #local ppHatScaleFactor     = 0.85;
  #local ppPivotPoint         = ppSkullEndPos;
  #local ppRefVector          = ppEndRefVector;



  #debug concat("TotalTurnAngle: ",str(ppTotalTurnAngle,3,3),"\n")
  #debug concat("<0, 1,-1>: "     ,str(VAngleD(<0, 1,-1>,-y),3,3),"\n")
  #debug concat("<0, 1, 1>: "     ,str(VAngleD(<0, 1, 1>,-y),3,3),"\n")
  #debug concat("<0,-1,-1>: "     ,str(VAngleD(<0,-1,-1>,-y),3,3),"\n")
  #debug concat("<0,-1, 1>: "     ,str(VAngleD(<0,-1, 1>,-y),3,3),"\n")
  #debug concat("<0, 1, 0>: "     ,str(VAngleD(<0, 1, 0>,-y),3,3),"\n")



  #if (ppVerbose = 1) #debug concat("Start of turn for droopy bit of Bobble Hat","\n")  #end  
  #local ppI = 0;
  #while (ppI<ppTurnSlices)
    #local ppHatVector = vaxis_rotate(ppHatVector,ppRotationAxisVector,ppDeltaAngle);
    #local ppRefVector  = vaxis_rotate(ppRefVector,ppRotationAxisVector,ppDeltaAngle);
    #local ppHatScaleFactor = ppHatScaleFactor + (1-ppHatScaleFactor)*0.04;
    #local ppPivotPoint = ppPivotPoint+ppDeltaDistance*ppHatVector; 
    #local ppJ = 0;
    #local ppK = ppHatSlices - 1;
    #while (ppJ < ppSegments)
      #declare ppHatArray [ppHatSlices][ppJ] = <1,ppHatScaleFactor,1>*(vaxis_rotate(ppHatArray[ppK][ppJ]-ppPivotPoint,ppRotationAxisVector,ppDeltaAngle))+ppPivotPoint+ppDeltaDistance*ppHatVector;
//      #declare ppHatNormal[ppHatSlices][ppJ] = ppHatNormal[ppK][ppJ]; 
      #local ppJ = ppJ + 1;
    #end
    #local ppHatSlices  = ppHatSlices  + 1; 
    #local ppI = ppI + 1;
  #end



//  #declare MyLine = union {
//    cylinder {ppSkullEndPos,ppSkullEndPos+ppEndRefVector,0.002 pigment {color rgb <1,1,0>}}
//    cylinder {ppSkullEndPos,ppSkullEndPos+ppSkullEndVector,0.002 pigment {color rgb <1,0,1>}}
//    sphere {ppSkullEndPos,0.002 pigment {color rgb <0,1,1>}}
//  }


  #local ppDropSlices = 5;
  #local ppDeltaDistance = 0.02; 
  #local ppHatVector = -y;

  #if (ppVerbose = 1) #debug concat("Start of drooping bit at the back of the Bobble Hat","\n")  #end  
  #local ppI = 0;
  #while (ppI<ppDropSlices)
    #local ppHatScaleFactor = 0.85;
    #local ppJ = 0;
    #local ppK = ppHatSlices - 1;
    #while (ppJ < ppSegments)
      #declare ppHatArray [ppHatSlices][ppJ] = <ppHatScaleFactor,1,ppHatScaleFactor>*ppHatArray[ppK][ppJ]+ppDeltaDistance*ppHatVector;
//      #declare ppHatNormal[ppHatSlices][ppJ] = ppHatNormal[ppK][ppJ]; 
      #local ppJ = ppJ + 1;
    #end
    #local ppHatSlices  = ppHatSlices  + 1; 
    #local ppI = ppI + 1;
  #end

//#local TempPoint1 = ppPivotPoint+ppDeltaDistance*ppHatVector*1;
//#local TempPoint2 = ppPivotPoint+ppDeltaDistance*ppHatVector*2;
//#local TempPoint3 = ppPivotPoint+ppDeltaDistance*ppHatVector*3;
//#local TempPoint4 = ppPivotPoint+ppDeltaDistance*ppHatVector*4;
//#local TempPoint5 = ppPivotPoint+ppDeltaDistance*ppHatVector*5;
//  #declare MyLine = union {
//    cylinder {ppHatArray[ppHatSlices-1][ppSegments/2],ppHatArray[ppHatSlices-1][ppSegments/2]+ppRefVector,0.002 pigment {color rgb <1,1,0>}}
//    cylinder {TempPoint1,TempPoint1+ppRefVector,0.002 pigment {color rgb <0,1,1>}}
//    cylinder {TempPoint2,TempPoint2+ppRefVector,0.002 pigment {color rgb <0,1,1>}}
//    cylinder {TempPoint3,TempPoint3+ppRefVector,0.002 pigment {color rgb <0,1,1>}}
//    cylinder {TempPoint4,TempPoint4+ppRefVector,0.002 pigment {color rgb <0,1,1>}}
//    cylinder {TempPoint5,TempPoint5+ppRefVector,0.002 pigment {color rgb <0,1,1>}}
//    cylinder {ppPivotPoint,ppPivotPoint+ppRefVector,0.002 pigment {color rgb <1,0,1>}}
//  }

  #if (ppVerbose = 1) #debug concat("Start of Bobble for Bobble Hat","\n")  #end  
  #local ppBobbleCentre = (ppHatArray[ppHatSlices-1][0]+ppHatArray[ppHatSlices-1][ppSegments/2])/2;
  #local ppBobbleRadius = 0.05;
  #local ppBobbleTexture = texture {
    pigment {color rgb <2,2,2>}
    normal {agate scale 0.01}
  }
  #local ppBobble = sphere {ppBobbleCentre,ppBobbleRadius texture{ppBobbleTexture}}

  #if (ppVerbose = 1) #debug concat("Start of Fringe for Bobble Hat","\n")  #end  
  #local ppFringeRadius = 0.05;  
  #local ppJ = 0;
  #local ppFringeTexture = texture {
    pigment {color rgb <2,2,2>}
    normal {agate scale 0.01}
  }
  #local ppXRotationAngle = 360/(ppFringeSlices-1);
  #local ppZRotationAngle = 360/ppSegments;
  #while (ppJ<ppSegments) 
    #local ppI = 0;
    #while (ppI < ppFringeSlices)
      #local ppFringeArray[ppI][ppJ] = vrotate(y*ppFringeRadius, <ppXRotationAngle*ppI,0,ppZRotationAngle*ppJ>)+ppHatArray[0][ppJ];
      #local ppI = ppI + 1; 
    #end
//    #if (ppJ = 0) #local ppFringe = sphere {ppHatArray[0][0],ppFringeRadius texture{ppFringeTexture}}
//    #else #local ppFringe = union {object {ppFringe} sphere {ppHatArray[0][ppJ],ppFringeRadius texture{ppFringeTexture}}}
//    #end
    #local ppJ = ppJ + 1;
  #end
  #declare ppFringe = object {ppMesh2FromArray(ppFringeArray,ppEmptyArray,ppFringeSlices,ppSegments,ppJoinEnds) texture {ppFringeTexture}} 



  #declare ppHat = ppMesh2FromArray(ppHatArray,ppEmptyArray,ppHatSlices,ppSegments,ppJoinEnds) 

  #if (ppVerbose = 1) #debug concat("Assemble Bobble Hat","\n")  #end  
  #declare ppHat = union {
    object {ppHat} 
    object {ppBobble}
    object {ppFringe}
  }
  
  #local ppShowPoints = 0;
  #if (ppShowPoints=1) ppDrawPoints(ppHatArray,ppSkullSlices,ppSegments,0.001,<2,0,0>) #end 



//  #declare ppHat = union {
//    object {ppHatLine    texture {ppHatTexture}}  
//  }
// 

#declare ppHat = object {ppHat texture {ppHatTexture}} 
#end

ppHeadwear()