/***************************************************************************************************/
/* pps2face.inc  - Copyright Chris Bartlett 2005                                                   */
/* This work is licensed for royalty free re-use under the terms of the POV-Person License.        */
/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */
/***************************************************************************************************/

#include "functions.inc"

// Define an isosurface to represent the shape of the upper skull
#local ppSkullCrownFunction = function {-f_superellipsoid(x,y,z-0.5, 0.85, 1)}
#local ppSkullBrowFunction  = function {-f_superellipsoid(x/0.9,(y-0.22)/0.75,z, 0.8, 0.8)}
#local ppBlob_threshold=0.01;

#declare ppSkullShape = isosurface {
  function {
    (1+ppBlob_threshold)
    -pow(ppBlob_threshold, ppSkullCrownFunction(x,y,z))
    -pow(ppBlob_threshold, ppSkullBrowFunction(x,y,z))
  }
  max_gradient 20
  contained_by { box { -2, 2 } }
}

// Build up the whole of santa's head  
#declare ppHeadShape = merge { 
  // Skull
  object {ppSkullShape  
    texture {ppSkinTexture}
    rotate x*15
    translate <0,0.2,0.1>
  } 
  // Chin
  sphere {0,0.39
    scale <2.4,3.3,1.4>
    rotate x*37
    translate < 0,-0.7,-0.4> 
    texture {ppSkinTexture}
  }
  // Jaw Line
  sphere {0,0.4
    scale <2.7,2.9,2>
    rotate x*30
    translate < 0,-0.36,0> 
    texture {ppSkinTexture}
  } 
  // Mouth
  sphere {0,0.4
    scale <1.4,1.2,1> 
    translate <0,-0.68,-0.715>
    texture {ppSkinTexture}
  }
}

// Define the line that the lips will follow
#declare ppLipSlices   = 9;
#declare ppUpperLipSegments = 4;
#declare ppUpperLipArray = array [ppLipSlices][ppUpperLipSegments];
#declare ppUpperLipArray [0][0] = <-0.42,0.1 ,0>;
#declare ppUpperLipArray [0][1] = <-0.3 ,0.09,0>;
#declare ppUpperLipArray [0][2] = <-0.09,0.1 ,0>;
#declare ppUpperLipArray [0][3] = < 0   ,0.05,0>;

#declare ppLowerLipSegments = 5;
#declare ppLowerLipArray = array [ppLipSlices][ppLowerLipSegments];
#declare ppLowerLipArray [0][0] = <-0.42, 0.1  ,0>;
#declare ppLowerLipArray [0][1] = <-0.25,-0.025,0>;
#declare ppLowerLipArray [0][2] = <-0.17,-0.06 ,0>;
#declare ppLowerLipArray [0][3] = <-0.07,-0.08 ,0>; 
#declare ppLowerLipArray [0][4] = < 0   ,-0.078,0>; 

// Trace a line from each upper lip positition back until it hits the head, then record the position back into the array
#local ppRayDirection = z;
#local ppI = 0;
#while (ppI < ppUpperLipSegments)
  #local ppHitNormal = <0,0,0>;
  #local ppRayStart = ppUpperLipArray[0][ppI]-<0,0.6,3>;
//cylinder {ppRayStart,ppRayStart+ppRayDirection*10,0.005 pigment {color rgb <0,1,1>}}
  #local ppRayHitPos = trace (ppHeadShape, ppRayStart,ppRayDirection,ppHitNormal);
  #if (vlength(ppHitNormal)!=0)          // i.e. If we hit the surface  
    #declare ppUpperLipArray [0][ppI] = ppRayHitPos;
//    #declare ppUpperLipNormal[0][ppI] = ppHitNormal;
//sphere {ppUpperLipArray[0][ppI],0.005 pigment {color rgb <0,1,1>}}
  #else #debug concat("Upper Lip Trace ",str(ppI,3,3)," Missed face\n")
  #end
  #local ppI = ppI + 1;
#end

// Trace a line from each lower lip positition back until it hits the head, then record the position back into the array
#local ppI = 0;
#while (ppI < ppLowerLipSegments)
  #local ppHitNormal = <0,0,0>;
  #local ppRayStart = ppLowerLipArray[0][ppI]-<0,0.6,3>;
//cylinder {ppRayStart,ppRayStart+ppRayDirection*10,0.005 pigment {color rgb <0,1,1>}}
  #local ppRayHitPos = trace (ppHeadShape, ppRayStart,ppRayDirection,ppHitNormal);
  #if (vlength(ppHitNormal)!=0)          // i.e. If we hit the surface  
    #declare ppLowerLipArray [0][ppI] = ppRayHitPos;
//    #declare ppLowerLipNormal[0][ppI] = ppHitNormal;
//sphere {ppLowerLipArray[0][ppI],0.005 pigment {color rgb <0,1,1>}}
  #else #debug concat("Lower Lip Trace ",str(ppI,3,3)," Missed face\n")
  #end
  #local ppI = ppI + 1;
#end

// The Mouth Line prism will be used to punch a hole through the face where the lips will go
#declare ppMouthLine = prism {
  linear_spline
  0, 1, 2*(ppUpperLipSegments+ppLowerLipSegments),
  // Upper lip left hand side moving from left to right 
  #local ppI = 0;
  #while (ppI < ppUpperLipSegments)
    <ppUpperLipArray[0][ppI].x,ppUpperLipArray[0][ppI].y>, 
    #local ppI = ppI + 1;
  #end 
  // Upper lip right hand side moving from left to right (mirror of left hand side) 
  #local ppI = ppUpperLipSegments - 1;
  #while (ppI >= 0)
    <-ppUpperLipArray[0][ppI].x,ppUpperLipArray[0][ppI].y>, 
    #local ppI = ppI - 1;
  #end 
  // Lower lip right hand side moving from right to left (mirror of left hand side) 
  #local ppI = 0;
  #while (ppI < ppLowerLipSegments)
    <-ppLowerLipArray[0][ppI].x,ppLowerLipArray[0][ppI].y>, 
    #local ppI = ppI + 1;
  #end 
  // Lower lip left hand side moving from right to left 
  #local ppI = ppLowerLipSegments - 1;
  #while (ppI >= 0)
    #if (ppI = 0) <ppLowerLipArray[0][ppI].x,ppLowerLipArray[0][ppI].y> 
    #else         <ppLowerLipArray[0][ppI].x,ppLowerLipArray[0][ppI].y>,
    #end 
    #local ppI = ppI - 1;
  #end
  rotate -x*90
  translate <0,0,-0.8>
}


// Create a shell for the head and punch a hole through for the mouth
#declare ppHeadShell = difference {
  object {ppHeadShape}
  object {ppHeadShape scale 0.999999}
  object {ppMouthLine}
}

// Build the face
#declare ppFace = union {
  // Tip of Nose
  sphere {<0,-0.23,-1.15>,0.2 
    pigment {rgb <1,0.4,0.4>}
  }
  // Bridge of Nose 
  sphere {0,0.2
    scale <0.7,2.8,1>
    rotate x*52 
    translate <0,0.12,-0.78>
    texture {ppSkinTexture}
  } 
  sphere {0,0.5 scale <1,0.8,1> translate <-0.4,-0.33,-0.44> pigment {rgb <1,0.7,0.7>}} // Right Cheek
  sphere {0,0.5 scale <1,0.8,1> translate < 0.4,-0.33,-0.44> pigment {rgb <1,0.7,0.7>}} // Left Cheek  
  object {ppHeadShell} 
}


//sphere {<-0.42,0,-0.65>,0.17 
//  pigment {rgb <1,1,1>}
//}
//sphere {< 0.42,0,-0.65>,0.17 
//  pigment {rgb <1,1,1>}
//}


//sphere {< 0.2,-0.9,-0.65>,0.35 pigment {rgb <1,0.9,0.9>}} // Chin Left
//sphere {<-0.2,-0.9,-0.65>,0.35 pigment {rgb <1,0.9,0.9>}} // Chin Right



//#declare ppSkullShape = isosurface {
//  function {pow(x,2) + pow(y,2) + pow(z,2) - 0.14}
////  threshold 0.14 
//  scale 0.5
//}



